/**
 * @file linear.c
 * @brief Contains linear split and placement functions for the Treemap visualization.
 *
 * This file implements helper functions to create nodes, perform linear splits,
 * place the nodes based on provided directions, and paint the resulting tree.
 */

#include "treemap.h" // Shared definitions.
#include "place.h"   // Placement-related functions.
/**
 * @brief Creates a new TreeNode instance.
 *
 * Allocates memory for a new TreeNode and initializes its values. If data is provided,
 * it sets the node sum to the value contained in the data.
 *
 * @param data Pointer to the data to be associated with the new node.
 * @return TreeNode* Pointer to the newly created node.
 */

// Helper Functions
TreeNode *create_node(Data *data) {
    TreeNode *node = malloc(sizeof(TreeNode));
    node->sum = data ? data->number : 0.0;
    node->children = NULL;
    node->num_children = 0;
    node->data = data;
    return node;
}

/**
 * @brief Performs a linear split of an array of data elements.
 *
 * Takes a segment of the data array and creates child nodes for each element in the range.
 *
 * @param array Pointer to the data array to split.
 * @param start Starting index of the segment to split.
 * @param end Ending index of the segment to split.
 * @return TreeNode* Pointer to the root node of the created subtree.
 */

// Linear Split
TreeNode *linear_split(Data *array, int start, int end) {
    if (start > end) return NULL;

    int size = end - start + 1;
    TreeNode *node = create_node(NULL);
    node->num_children = size;
    node->children = malloc(size * sizeof(TreeNode *));
    node->sum = 0.0;

    for (int i = start; i <= end; i++) {
        node->children[i - start] = create_node(&array[i]);
        node->sum += array[i].number;
    }

    return node;
}
/**
 * @brief Places the nodes of the tree in a given direction recursively.
 *
 * Assigns positions (bounding boxes) to nodes according to the provided direction sequence
 * and recursively places children nodes.
 *
 * @param node Pointer to the root of the subtree to place.
 * @param directions A string containing direction instructions (e.g., "WNES").
 * @param depth Current depth in the tree (used for determining the direction).
 */

// Place Function
void place(TreeNode *node, const char *directions, int depth) {
    if (!node || node->num_children == 0) return;

    int dir_length = strlen(directions);
    char direction = directions[depth % dir_length];

    double x = node->box.x;
    double y = node->box.y;
    double w = node->box.w;
    double h = node->box.h;

    for (int i = 0; i < node->num_children; i++) {
        TreeNode *child = node->children[i];
        double ratio = child->sum / node->sum;

        switch (direction) {
            case 'W':
                child->box = (Box){x, y, ratio * w, h};
                x += ratio * w;
                break;
            case 'N':
                child->box = (Box){x, y, w, ratio * h};
                y += ratio * h;
                break;
            case 'E':
                x += w * (1 - ratio);
                child->box = (Box){x, y, ratio * w, h};
                w *= (1 - ratio);
                break;
            case 'S':
                y += h * (1 - ratio);
                child->box = (Box){x, y, w, ratio * h};
                h *= (1 - ratio);
                break;
            default:
                fprintf(stderr, "Invalid direction: %c\n", direction);
                exit(EXIT_FAILURE);
        }

        place(child, directions, depth + 1);
    }
}
/**
 * @brief Paints the tree by printing the bounding boxes and associated labels.
 *
 * This function prints the coordinates and dimensions of the bounding boxes
 * of leaf nodes along with their labels.
 *
 * @param node Pointer to the root of the subtree to paint.
 */
// Paint Function
void paint(TreeNode *node) {
    if (!node) return;

    if (node->data) {
        double scaled_x = node->box.x * 100;
        double scaled_y = node->box.y * 100;
        double scaled_w = node->box.w * 100;
        double scaled_h = node->box.h * 100;

        printf("%.2f:%.2f:%.2f:%.2f:%s\n",
               scaled_x,
               scaled_y,
               scaled_w,
               scaled_h,
               node->data->label ? node->data->label : "unknown");
    } else {
        for (int i = 0; i < node->num_children; i++) {
            paint(node->children[i]);
        }
    }
}

/**
 * @brief Frees the memory allocated for a tree.
 *
 * This function recursively frees all nodes in a given tree, including
 * any dynamically allocated child nodes and associated data.
 *
 * @param node Pointer to the root of the subtree to free.
 */

void free_tree(TreeNode *node) {
    if (node == NULL) return;

    // Free children
    for (int i = 0; i < node->num_children; i++) {
        free_tree(node->children[i]);
    }

    // Free the array of children and the node itself
    free(node->children);
    free(node);
}

/**
 * @brief Main function to demonstrate the linear split and placement.
 *
 * Reads data from a file, builds a treemap structure, places the nodes,
 * and paints them to the console.
 *
 * @return int Returns 0 if successful.
 */

// Main Function
int main() {
    int capacity = 5;
    int current_size = 0;
    Data *array = malloc(capacity * sizeof(Data));

    FILE *file = fopen("plain.txt", "r");
    if (file == NULL) {
        perror("File open error");
        free(array);
        return 1;
    }

    double number;
    char name[MAX_NAME_SIZE];
    while (fscanf(file, "%lf %s", &number, name) == 2) {
        if (current_size >= capacity) {
            capacity *= 2;
            array = realloc(array, capacity * sizeof(Data));
        }
        array[current_size].number = number;
        array[current_size].label = strdup(name);
        current_size++;
    }
    fclose(file);

    TreeNode *root = linear_split(array, 0, current_size - 1);
    root->box = (Box){0, 0, 1, 1};
    place(root, "WNES", 0);
    paint(root);

    for (int i = 0; i < current_size; i++) {
        free(array[i].label);
    }
    free(array);
    free_tree(root);

    return 0;
}

