/**
 * @file place.c
 * @brief Implements the node placement logic for the TreeMap visualization.
 *
 * This file contains functions to create nodes, build the tree structure, place the nodes,
 * and display the assigned bounding boxes for each node.
 */


/**
 * @brief Creates a new TreeNode instance.
 *
 * Allocates memory for a new TreeNode and initializes its values. If data is provided,
 * it sets the node sum to the value contained in the data.
 *
 * @param data Pointer to the data to be associated with the new node.
 * @return TreeNode* Pointer to the newly created node.
 */
TreeNode *create_node(Data *data) {
    TreeNode *node = malloc(sizeof(TreeNode));
    node->sum = data ? data->number : 0.0;
    node->children = NULL;
    node->num_children = 0;
    node->data = data;
    return node;
}

/**
 * @brief Builds a binary tree from an array of data elements.
 *
 * This function takes an array of data and recursively divides it to create a binary tree.
 * Internal nodes have a sum of the values of their child nodes.
 *
 * @param array Pointer to the data array.
 * @param start The starting index for the current segment.
 * @param end The ending index for the current segment.
 * @return TreeNode* Pointer to the root node of the created subtree.
 */

TreeNode *build_tree(Data *array, int start, int end) {
    if (start > end) return NULL;

    // Cas d'une feuille (un seul élément)
    if (start == end) {
        return create_node(&array[start]);
    }

    // Calculer la division
    int size = end - start + 1;
    int mid = start + (size + 1) / 2;

    // Créer un nœud interne
    TreeNode *node = create_node(NULL);

    // Ajouter les enfants récursivement
    node->children = malloc(2 * sizeof(TreeNode *));
    node->num_children = 2;

    node->children[0] = build_tree(array, start, mid - 1);
    node->children[1] = build_tree(array, mid, end);

    // Calculer la somme des valeurs des enfants
    node->sum = 0.0;
    for (int i = 0; i < node->num_children; i++) {
        if (node->children[i]) {
            node->sum += node->children[i]->sum;
        }
    }

    return node;
}

/**
 * @brief Displays the tree structure with indentation to indicate depth.
 *
 * This function recursively traverses the tree and prints each node's details,
 * including its sum and, if applicable, its data.
 *
 * @param node Pointer to the root of the subtree to display.
 * @param depth The current depth for indentation.
 */

void display_tree(TreeNode *node, int depth) {
    if (node == NULL) return;

    // Indentation pour représenter la profondeur
    for (int i = 0; i < depth; i++) {
        printf("  ");
    }

    if (node->data) {
        // Afficher une feuille
        printf("Leaf: {%.0f, \"%s\"}\n", node->data->number, node->data->label ? node->data->label : "");
    } else {
        // Afficher un nœud interne
        printf("Node (sum: %.0f)\n", node->sum);
    }

    // Afficher les enfants
    for (int i = 0; i < node->num_children; i++) {
        display_tree(node->children[i], depth + 1);
    }
}
/**
 * @brief Frees the memory allocated for a tree.
 *
 * This function recursively frees all nodes in a given tree, including
 * any dynamically allocated child nodes.
 *
 * @param node Pointer to the root of the subtree to free.
 */

void free_tree(TreeNode *node) {
    if (node == NULL) return;

    // Libérer les enfants
    for (int i = 0; i < node->num_children; i++) {
        free_tree(node->children[i]);
    }

    // Libérer le tableau des enfants et le nœud lui-même
    free(node->children);
    free(node);
}


/**
 * @brief Places nodes in a treemap visualization by assigning bounding boxes.
 *
 * This function assigns bounding boxes to nodes in the tree based on the given direction sequence.
 * Each child node is given a proportion of the parent's bounding box.
 *
 * @param node Pointer to the root of the subtree to place.
 * @param directions A string containing direction instructions (e.g., "WNES").
 * @param depth The current depth in the tree (used for determining the direction).
 */


void place(TreeNode *node, const char *directions, int depth) {
    if (!node || node->num_children == 0) return;

    // Determine direction for this depth
    int dir_length = strlen(directions);
    char direction = directions[depth % dir_length]; // Wrap around if depth exceeds string length

    // Starting position for children
    double x = node->box.x;
    double y = node->box.y;
    double w = node->box.w;
    double h = node->box.h;

    for (int i = 0; i < node->num_children; i++) {
        TreeNode *child = node->children[i];
        double ratio = child->sum / node->sum; // Relative size of the child box

        switch (direction) {
            case 'W': // Place left-to-right
                child->box = (Box){x, y, ratio * w, h};
                x += ratio * w;
                break;

            case 'N': // Place top-to-bottom
                child->box = (Box){x, y, w, ratio * h};
                y += ratio * h;
                break;

            case 'E': // Place right-to-left
                x += w * (1 - ratio); // Start at the right edge
                child->box = (Box){x, y, ratio * w, h};
                w *= (1 - ratio); // Adjust width
                break;

            case 'S': // Place bottom-to-top
                y += h * (1 - ratio); // Start at the bottom
                child->box = (Box){x, y, w, ratio * h};
                h *= (1 - ratio); // Adjust height
                break;

            default:
                fprintf(stderr, "Invalid direction: %c\n", direction);
                exit(EXIT_FAILURE);
        }

        // Recursively place children of the current child
        place(child, directions, depth + 1);
    }
}

/**
 * @brief Displays the bounding boxes of all nodes in the tree.
 *
 * This function traverses the tree and prints out the details of each node's assigned bounding box.
 *
 * @param node Pointer to the root of the subtree to display.
 * @param depth The current depth for indentation.
 */

void display_boxes(TreeNode *node, int depth) {
    if (!node) return;

    // Indentation for depth
    for (int i = 0; i < depth; i++) {
        printf("  ");
    }

    // Display the box information
    if (node->data) {
        printf("Leaf: {%.0f, \"%s\"} -> Box(x=%.2f, y=%.2f, w=%.2f, h=%.2f)\n",
               node->data->number,
               node->data->label ? node->data->label : "",
               node->box.x, node->box.y, node->box.w, node->box.h);
    } else {
        printf("Node (sum: %.0f) -> Box(x=%.2f, y=%.2f, w=%.2f, h=%.2f)\n",
               node->sum, node->box.x, node->box.y, node->box.w, node->box.h);
    }

    // Display children
    for (int i = 0; i < node->num_children; i++) {
        display_boxes(node->children[i], depth + 1);
    }
}

