#include "treemap.h"
#include "split-binary.c"
#include "place.h"
#include "place.c"
#include "paint.c"

/**
 * @brief Adds a new data element to the dynamic array.
 * 
 * If the array reaches its capacity, it will automatically resize itself to hold more elements.
 * This function is used to manage the data elements we read from the input.
 * 
 * @param array Pointer to the dynamic array that stores data elements.
 * @param current_size Pointer to the current number of elements in the array.
 * @param capacity Pointer to the current capacity of the array.
 * @param number The numerical value of the new element to add.
 * @param label The label for the new element. This can be NULL if there's no label.
 */


// LA PARTIE INPUT
// Function to add an element to the dynamic array
void add_data(Data **array, int *current_size, int *capacity, double number, const char *label) {
    // Resize the array if necessary
    if (*current_size >= *capacity) {
        *capacity *= 2;
        *array = realloc(*array, *capacity * sizeof(Data));
        if (*array == NULL) {
            perror("Memory allocation error");
            exit(EXIT_FAILURE);
        }
    }

// SPLIT SECTION
// LINEAR SPLIT
    // Add the element to the array
    (*array)[*current_size].number = number;
    if (label != NULL) {
        (*array)[*current_size].label = malloc((strlen(label) + 1) * sizeof(char));
        strcpy((*array)[*current_size].label, label);
    } else {
        (*array)[*current_size].label = NULL;
    }
    (*current_size)++;
}

/**
 * @brief Displays all the elements in the array.
 * 
 * This function prints out all the data elements in the array in a readable format.
 * 
 * @param array Pointer to the dynamic array.
 * @param size The current number of elements in the array.
 */


// Function to display the array
void display_array(Data *array, int size) {
    printf("[");
    for (int i = 0; i < size; i++) {
        printf("{%.0f, \"%s\"}", array[i].number, array[i].label ? array[i].label : "");
        if (i < size - 1) {
            printf(", ");
        }
    }
    printf("]\n");
}

/**
 * @brief Frees the memory allocated for the dynamic array.
 * 
 * This function goes through all elements in the array and frees any dynamically allocated memory.
 * 
 * @param array Pointer to the dynamic array.
 * @param size The number of elements in the array.
 */


// Function to free the memory of the array
void free_array(Data *array, int size) {
    for (int i = 0; i < size; i++) {
        free(array[i].label);  // Free each allocated label
    }
    free(array);  // Free the array itself
}

/**
 * @brief Main function that manages the TreeMap process.
 * 
 * This function initializes the data, reads from input, performs the binary split, and then places elements in a tree.
 * 
 * @return int Returns 0 if the program completes successfully.
 */


int main() {
    // PARTIE INPUT
    // Declaration and initialization of variables for the dynamic array
    int capacity = 5;
    int current_size = 0;
    Data *array = malloc(capacity * sizeof(Data));

    if (array == NULL) {
        perror("Memory allocation error");
        return 1;
    }

    // Open the file in read mode
    FILE *file = fopen("plain.txt", "r");
    if (file == NULL) {
        perror("File open error");
        free(array);
        return 1;
    }

    // Read each line from the file and add to the array
    double number;
    char name[MAX_NAME_SIZE];
    while (fscanf(file, "%lf %s", &number, name) == 2) {
        add_data(&array, &current_size, &capacity, number, name);
    }

    fclose(file);  // Close the file
    // END OF INPUT
    
    // RESULTS
    // Display the array of data
    //display_array(array, current_size);
    
    // BINARY SPLIT
    //printf("\nBinary split results:\n");
    binary_split(array, 0, current_size - 1);
    // FIN BINARY SPLIT
    // Build the tree
    TreeNode *root = build_tree(array, 0, current_size - 1);

    // Assign the root's initial box
    root->box = (Box){0, 0, 1, 1};  // Initial box for the root node

    // Place nodes with "WNES" directions
    place(root, "WNES", 0);

    // Display results
    //printf("\nTree with assigned boxes:\n");
    //display_boxes(root, 0);
    //printf("Résultats de paint:\n");
    paint(root);


    // Free allocated memory
    free_tree(root);  // Recursively free the tree
    free_array(array, current_size);

    return 0;
}

